/*
 *  AUCarbonViewNib.cpp
 *  AUNib
 *
 *  Created by Airy Andr on Sun Oct 06 2002.
 *  Copyright (c) 2002 __MyCompanyName__. All rights reserved.
 *
 */

#include "AUCarbonViewNib.h"
#include "AUControlGroup.h"
#include "TPngPictControl.h"
#include "TValuePictControl.h"
#include "TKnobControl.h"

#include "TImageCache.h"

// -----------------------------------------------------------------------------
// Class CAUCarbonViewNib
//
// This class generates a custom view of the plugin from a nib window.
// -----------------------------------------------------------------------------
static pascal void BackgroundEraseRectProc( GrafVerb verb, const Rect* r )
{
    if( verb != kQDGrafVerbErase ) {
	// not interested
	StdRect( verb, r );
    } else {
	CAUCarbonViewNib *view = 0;
	
	StdRect( verb, r );
	CGrafPtr currPort;
	GetPort( &currPort);
	WindowRef theWin = GetWindowFromPort(currPort);
	if (theWin) {
	    GetWindowProperty(theWin, 'AIRY', 'CVNB', sizeof(CAUCarbonViewNib *), 0, &view);
	}
	if (view && view->GetBackgroundImage()) {
	    CGContextRef context;
	    Rect rect;
	    
	    QDBeginCGContext(currPort, &context);
	    GetPortBounds(currPort, &rect);
	    CGContextTranslateCTM(context, 0, float(rect.bottom-rect.top));
	    CGContextScaleCTM(context, 1, -1);

	    TRect trect = *r;
	    CGContextClipToRect(context, trect);

	    Rect paneRect;
	    view->GetPaneBounds(&paneRect);
	    trect = paneRect;
	    HIViewDrawCGImage(context, &trect, view->GetBackgroundImage());

	    CGContextSynchronize ( context );
	    QDEndCGContext (currPort, &context );
	}
   }
}
// -----------------------------------------------------------------------------
CAUCarbonViewNib::CAUCarbonViewNib(AudioUnitCarbonView tView):
AUCarbonViewBase(tView), mCurrentProcsPtr(0), mBackgroundImage(0)
{
}
// -----------------------------------------------------------------------------
CAUCarbonViewNib::~CAUCarbonViewNib()
{
    if (mBackgroundImage) {
	SetPortGrafProcs( GetWindowPort( mCarbonWindow ), mCurrentProcsPtr);
	DisposeRoutineDescriptor (mProcs.rectProc);
	CGImageRelease(mBackgroundImage);
    }
    // We should unregister control classes here...
    TImageCache::Reset();
}
// -----------------------------------------------------------------------------
void CAUCarbonViewNib::InitWindow(CFBundleRef sBundle) // Register custom controls...
{
    HIViewRef		view;
    // Workaround: Unfortunately, at the time we call RegisterClass below,
    // the HIView base class isn't registered. It's supposed to be automatically
    // registered, but something is going wrong in HIToolbox. We can force
    // it to register by creating any arbitrary view. Here, we simply create
    // and release a scroll view. That's enough to make sure the HIView base
    // class is registered. Sorry folks.
    HIScrollViewCreate( kHIScrollViewOptionsVertScroll, &view );
    CFRelease( view );
}
// -----------------------------------------------------------------------------
void CAUCarbonViewNib::FinishWindow(CFBundleRef sBundle)
{
}
// -----------------------------------------------------------------------------
OSStatus CAUCarbonViewNib::CreateUI(Float32 x, Float32 y)
{
    Rect nibFrame, controlFrame;
    WindowRef win;
    IBNibRef      inNibRef;
    CFBundleRef sBundle = NULL;

    sBundle = CFBundleGetBundleWithIdentifier (GetBundleID());
    CFRetain (sBundle);
    InitWindow(sBundle);
    
    CreateNibReferenceWithCFBundle(sBundle,
				   GetWindowNibName(),
				   &inNibRef);

    // Register standard SDK controls
   TPngPictControl::SetClassBundle(sBundle);
   TPngPictControl::RegisterClassForBundleID(GetBundleID());

   TValuePictControl::SetClassBundle(sBundle);
   TValuePictControl::RegisterClassForBundleID(GetBundleID());

   TKnobControl::SetClassBundle(sBundle);
   TKnobControl::RegisterClassForBundleID(GetBundleID());
    
    WindowAttributes attributes;
    GetWindowAttributes(mCarbonWindow, &attributes);
    bool hasCompositing = attributes & kWindowCompositingAttribute;
    if (hasCompositing)
	CreateWindowFromNib( inNibRef, GetWindowCompositingName(), &win);
    else
	CreateWindowFromNib( inNibRef, GetWindowNoCompositingName(), &win);
    
    if (HasBackgroundPict() && !hasCompositing) {
	CFURLRef url = ::CFBundleCopyResourceURL( sBundle, GetBackgroundFilename(), NULL, NULL );
	if (url) {
	    mBackgroundImage = TImageCache::GetImage(url);
	    CFRelease(url);
	    if (mBackgroundImage) {
		// Add a property to the Window so anybody with a window ptr can get the AUCarbonViewNib
		CAUCarbonViewNib *ptr = this;
		SetWindowProperty(mCarbonWindow, 'AIRY', 'CVNB', sizeof(CAUCarbonViewNib *), &ptr);

		if (PatchEraseRect()) {
		    mCurrentProcsPtr = GetPortGrafProcs( GetWindowPort( mCarbonWindow ) );
		    if ( mCurrentProcsPtr != NULL )
			mProcs = *mCurrentProcsPtr;
		    else
			SetStdCProcs( &mProcs );
		    mProcs.rectProc = NewQDRectUPP( BackgroundEraseRectProc );
		    SetPortGrafProcs( GetWindowPort( mCarbonWindow ), &mProcs );
		}
	    }
	}
    }
    if (mBackgroundImage == 0) {
	// set the background theme
	SetThemeWindowBackground(mCarbonWindow, kThemeBrushDialogBackgroundActive, false );
    }


    OSStatus result = noErr;
    ControlRef control;
#if 1
    ControlID controlID = { 'AUid', 9999 }; // The user pane to embed
    result = GetControlByID (win, &controlID, &control);
#else
    result = GetRootControl(win, &control); // Why this does not work ???
#endif
    if (result == noErr) {
	GetControlBounds (control, &controlFrame);
	OffsetRect (&controlFrame, (short)x, (short)y);
	SetControlBounds (control, &controlFrame);

	if (HasBackgroundPict()) {
	    HIViewRef backCtrl;
	    CFMutableStringRef reg = CFStringCreateMutableCopy(NULL, 100, GetBundleID());
	    CFStringAppend(reg, CFSTR(".pict"));
            TPngPictControl::Create(&backCtrl, 0, 0, reg);
	    CFRelease(reg);
	    SetControlTitleWithCFString(backCtrl, GetBackgroundFilename()) ;
	    SetControlBounds (backCtrl, &controlFrame);
	    HIViewAddSubview( mCarbonPane, backCtrl);
	    HIViewAddSubview( backCtrl, control );
	    ShowControl( backCtrl );
	} else {
	    HIViewAddSubview( mCarbonPane, control);
	}
    }
    GetWindowBounds (win, kWindowContentRgn, &nibFrame);
    SizeControl(mCarbonPane, nibFrame.right-nibFrame.left, nibFrame.bottom-nibFrame.top);
    SizeWindow(mCarbonWindow, nibFrame.right-nibFrame.left, nibFrame.bottom-nibFrame.top, TRUE);
    ReleaseWindow(win);

    UInt32 size;
    Boolean writable;
    AudioUnitParameterID *ids;
    AudioUnitGetPropertyInfo(mEditAudioUnit,
			     kAudioUnitProperty_ParameterList,
			     kAudioUnitScope_Global,
			     0,
			     &size,
			     &writable);
    ids = new AudioUnitParameterID[size/sizeof(AudioUnitParameterID)];
    AudioUnitGetProperty(mEditAudioUnit,
			     kAudioUnitProperty_ParameterList,
			     kAudioUnitScope_Global,
			     0,
			     ids,
			     &size);
    
    for (unsigned int i = 0; i <  size/sizeof(AudioUnitParameterID); ++i) {
	AUVParameter tParam(mEditAudioUnit, ids[i], kAudioUnitScope_Global, 0);
	ControlKind kind;
	ControlRef control;
        AUCarbonViewControl::ControlType controlType;
	for (int j = 0; j < 20; ++j) {
	    ControlID controlID = { 'AUid', j*1000+ids[i] };
	    if (GetControlByID (mCarbonWindow, &controlID, &control) == noErr) {
		// Bind control to parameter i
		GetControlKind(control, &kind);
		switch (kind.kind) {
		    case kControlKindCheckBox:
		    case kControlKindRadioGroup: {
			controlType = AUCarbonViewControl::kTypeDiscrete;
			break;
		    }
		    case kControlKindEditText:
		    case kControlKindStaticText:
		    case kControlKindEditUnicodeText: {
			controlType = AUCarbonViewControl::kTypeText;
			break;
		    }
		    case kControlKindPopupArrow:
		    case kControlKindPopupButton:
			if (tParam.HasNamedParams() && GetControlByID (mCarbonWindow, &controlID, &control) == noErr) {
			    MenuRef menuRef;
			    CreateNewMenu( 1, 0, &menuRef);

			    for (int i = 0; i < tParam.GetNumIndexedParams(); ++i)
			    {
				AppendMenuItemTextWithCFString (menuRef, tParam.GetParamName(i), 0, 0, 0);
			    }

			    SetControlData(control, 0, kControlPopupButtonMenuRefTag, sizeof(menuRef), &menuRef);
			    SetControlMaximum(control, tParam.GetNumIndexedParams());
			    controlType = AUCarbonViewControl::kTypeDiscrete;
			}
			break;
		    default: {
			int mini, maxi;

			if (tParam.ParamInfo().unit == kAudioUnitParameterUnit_Indexed) {
			    controlType = AUCarbonViewControl::kTypeDiscrete;
			    mini = int(tParam.ParamInfo().minValue);
			    maxi = int(tParam.ParamInfo().maxValue);
			} else {
			    controlType = AUCarbonViewControl::kTypeContinuous;
			    mini = 0;
			    maxi = 10000;
			}
			SetControlMinimum(control, mini);
			SetControlMaximum(control, maxi);
			if (kind.kind == kControlKindSlider)
			    SetControlAction(control, AUCarbonViewControl::SliderTrackProc);
			break;
		    }
		}
		SetControlValue(control, (short)tParam.GetValue());
		AUCarbonViewControl *auvc = new AUCarbonViewControl(this, mParameterListener,
						      controlType, tParam, control);
		auvc->Bind();
		AddControl(auvc);
	    }
	}
    }
    delete[] ids;

    FinishWindow(sBundle);
    DisposeNibReference(inNibRef);
    CFRelease(sBundle);    
   return noErr;
}
// -----------------------------------------------------------------------------
void CAUCarbonViewNib::GetPaneBounds(Rect *r)
{
    GetControlBounds (mCarbonPane, r);
}
// -----------------------------------------------------------------------------
// End of file.
// -----------------------------------------------------------------------------
