/*
 *  TPngPictControl.cpp
 *  NeuSynth
 *
 *  Created by Airy Andr on Sun Oct 20 2002.
 *  Copyright (c) 2002 __MyCompanyName__. All rights reserved.
 *
 */

#include "TPngPictControl.h"
#include "TImageCache.h"


CFBundleRef TPngPictControl::sStaticBundle = 0;
// -----------------------------------------------------------------------------
//	TPngPictControl constructor
// -----------------------------------------------------------------------------
//
TPngPictControl::TPngPictControl(
			 HIViewRef			inControl )
:	TViewNoCompositingCompatible( inControl )
{
    ChangeAutoInvalidateFlags( kAutoInvalidateOnActivate | kAutoInvalidateOnEnable, 0 );
    mBundleRef = sStaticBundle;
    if (mBundleRef)
	CFRetain(mBundleRef);

    mImage = 0;
}

// -----------------------------------------------------------------------------
//	TPngPictControl destructor
// -----------------------------------------------------------------------------
//	Clean up after yourself.
//
TPngPictControl::~TPngPictControl()
{
    if (mImage)
	CGImageRelease(mImage);
    if (mBundleRef)
	CFRelease(mBundleRef);
}

// -----------------------------------------------------------------------------
//	SetClassBundle
// -----------------------------------------------------------------------------
//
void TPngPictControl::SetClassBundle(CFBundleRef sBundle)
{
    if (sStaticBundle)
	CFRelease(sBundle);

    if (sBundle)
	CFRetain (sBundle);
    sStaticBundle = sBundle;
}

// -----------------------------------------------------------------------------
//	GetKind
// -----------------------------------------------------------------------------
//
ControlKind TPngPictControl::GetKind()
{
    const ControlKind kMyKind = { 'airy', 'png ' };

    return kMyKind;
}

//-----------------------------------------------------------------------------------
//	Create
//-----------------------------------------------------------------------------------
//
OSStatus TPngPictControl::Create(
			      HIViewRef*			outControl,
			      const HIRect*		inBounds,
			      WindowRef			inWindow,
			      CFStringRef 	classID)
{
    OSStatus			err;
    EventRef			event = CreateInitializationEvent();
    ControlRef			root;

    RegisterClass(classID);
    err = HIObjectCreate( classID, event, (HIObjectRef*) outControl );

    ReleaseEvent( event );

    require_noerr( err, CantCreateHIObject );

    if ( inWindow != NULL )
    {
	GetRootControl( inWindow, &root );
	HIViewAddSubview( root, *outControl );
    }

    if ( inBounds != NULL )
	HIViewSetFrame( *outControl, inBounds );

CantCreateHIObject:
	return err;
}

//-----------------------------------------------------------------------------------
//	RegisterClassForBundleID
//-----------------------------------------------------------------------------------
//
void TPngPictControl::RegisterClassForBundleID(CFStringRef str)
{
    CFMutableStringRef reg = CFStringCreateMutableCopy(NULL, 100, str);
    CFStringAppend(reg, CFSTR(".pict"));
    TView::RegisterSubclass( reg, Construct);
    CFRelease(reg);
}
//-----------------------------------------------------------------------------------
//	RegisterClass
//-----------------------------------------------------------------------------------
//
void TPngPictControl::RegisterClass(CFStringRef str)
{
TView::RegisterSubclass( str, Construct);
}
//-----------------------------------------------------------------------------------
//	Construct
//-----------------------------------------------------------------------------------
//
OSStatus TPngPictControl::Construct(
				ControlRef		inControl,
				TView**			outView )
{
    *outView = new TPngPictControl( inControl );

    return noErr;
}

//-----------------------------------------------------------------------------------
//	Initialize
//-----------------------------------------------------------------------------------
//	The control is set up.  Do the last minute stuff that needs to be done like
//	installing EventLoopTimers.
//
OSStatus TPngPictControl::Initialize(
				 TCarbonEvent&		inEvent )
{
#pragma unused( inEvent )
    TitleChanged();
    return noErr;
}
//-----------------------------------------------------------------------------------
//	TitleChanged
//-----------------------------------------------------------------------------------
OSStatus TPngPictControl::TitleChanged()
{
    if (mImage)
	CGImageRelease(mImage);
    if ( mBundleRef )
    {
	CFStringRef fileName;
	CopyControlTitleAsCFString(GetViewRef(), &fileName);

	if ( fileName != NULL )
	{
	    CFURLRef url = ::CFBundleCopyResourceURL( mBundleRef, fileName, NULL, NULL );
	    if (url) {
		mImage = TImageCache::GetImage(url);
		CFRelease( url );
	    }
	    CFRelease( fileName );
	}
    }
    return noErr;
}

//-----------------------------------------------------------------------------------
//	Draw
//-----------------------------------------------------------------------------------
//	The fun part of the control
//
void TPngPictControl::CompatibleDraw(
				     RgnHandle				inLimitRgn,
				     CGContextRef			inContext,
				     bool  inCompositing)
{
#pragma unused( inLimitRgn )
    if (mImage) {
	HIRect bounds = Bounds();
	HIViewDrawCGImage( inContext, &bounds, mImage);
    }
}

//-----------------------------------------------------------------------------------
//	GetRegion
//-----------------------------------------------------------------------------------
//
OSStatus TPngPictControl::GetRegion(
				ControlPartCode		inPart,
				RgnHandle			outRgn )
{
    OSStatus			err = eventNotHandledErr;

    switch ( inPart )
    {
	case kControlContentMetaPart:
	case kControlStructureMetaPart:
	    TRect bounds = Bounds();
	    Rect qdBounds = bounds;
	    RectRgn( outRgn, &qdBounds );
	    err = noErr;
	    break;
    }

    return err;
}

UInt32 TPngPictControl::GetBehaviors()
{
    return TView::GetBehaviors() | kControlSupportsEmbedding | kControlHasSpecialBackground;
}