/*
 *  TValuePictControl.cpp
 *  NeuSynth
 *
 *  Created by Airy Andr on Sun Oct 20 2002.
 *  Copyright (c) 2002 __MyCompanyName__. All rights reserved.
 *
 */

#include "TValuePictControl.h"
#include "TImageCache.h"

CFBundleRef TValuePictControl::sStaticBundle = 0;
// -----------------------------------------------------------------------------
//	TValuePictControl constructor
// -----------------------------------------------------------------------------
//
TValuePictControl::TValuePictControl(
			 HIViewRef			inControl )
:	TViewNoCompositingCompatible( inControl )
{
    ChangeAutoInvalidateFlags( kAutoInvalidateOnActivate | kAutoInvalidateOnEnable, 0 );
    mBundleRef = sStaticBundle;
    if (mBundleRef)
	CFRetain(mBundleRef);

    mImage = 0;
    mImageCache = CFDictionaryCreateMutable(NULL, 0, &kCFTypeDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
}

// -----------------------------------------------------------------------------
//	TValuePictControl destructor
// -----------------------------------------------------------------------------
//	Clean up after yourself.
//
TValuePictControl::~TValuePictControl()
{
    if (mImage)
	CGImageRelease(mImage);
    if (mBundleRef)
	CFRelease(mBundleRef);
    if (mImageCache)
	CFRelease(mImageCache);
}

// -----------------------------------------------------------------------------
//	SetClassBundle
// -----------------------------------------------------------------------------
//
void TValuePictControl::SetClassBundle(CFBundleRef sBundle)
{
    if (sStaticBundle)
	CFRelease(sBundle);

    if (sBundle)
	CFRetain (sBundle);
    sStaticBundle = sBundle;
}

// -----------------------------------------------------------------------------
//	GetKind
// -----------------------------------------------------------------------------
//
ControlKind TValuePictControl::GetKind()
{
    const ControlKind kMyKind = { 'airy', 'valp' };

    return kMyKind;
}

//-----------------------------------------------------------------------------------
//	Create
//-----------------------------------------------------------------------------------
//
OSStatus TValuePictControl::Create(
			     HIViewRef*			outControl,
			     const HIRect*		inBounds,
				   WindowRef			inWindow,
				   CFStringRef 	classID)
{
    OSStatus			err;
    EventRef			event = CreateInitializationEvent();
    ControlRef			root;

    RegisterClass(classID);
    err = HIObjectCreate( classID, event, (HIObjectRef*) outControl );

    ReleaseEvent( event );

    require_noerr( err, CantCreateHIObject );

    if ( inWindow != NULL )
    {
	GetRootControl( inWindow, &root );
	HIViewAddSubview( root, *outControl );
    }

    if ( inBounds != NULL )
	HIViewSetFrame( *outControl, inBounds );

CantCreateHIObject:

	return err;
}

//-----------------------------------------------------------------------------------
//	RegisterClass
//-----------------------------------------------------------------------------------
//
void TValuePictControl::RegisterClass(CFStringRef str)
{
TView::RegisterSubclass( str, Construct);
}
//-----------------------------------------------------------------------------------
//	RegisterClassForBundleID
//-----------------------------------------------------------------------------------
//
void TValuePictControl::RegisterClassForBundleID(CFStringRef str)
{
    CFMutableStringRef reg = CFStringCreateMutableCopy(NULL, 100, str);
    CFStringAppend(reg, CFSTR(".valpict"));
    TView::RegisterSubclass( reg, Construct);
    CFRelease(reg);
}
//-----------------------------------------------------------------------------------
//	Construct
//-----------------------------------------------------------------------------------
//
OSStatus TValuePictControl::Construct(
				ControlRef		inControl,
				TView**			outView )
{
    *outView = new TValuePictControl( inControl );

    return noErr;
}

//-----------------------------------------------------------------------------------
//	Initialize
//-----------------------------------------------------------------------------------
//	The control is set up.  Do the last minute stuff that needs to be done like
//	installing EventLoopTimers.
//
OSStatus TValuePictControl::Initialize(
				 TCarbonEvent&		inEvent )
{
#pragma unused( inEvent )
    ValueChanged();
    return noErr;
}
//-----------------------------------------------------------------------------------
//	TitleChanged
//-----------------------------------------------------------------------------------
OSStatus TValuePictControl::TitleChanged()
{
    CFDictionaryRemoveAllValues(mImageCache);
    ValueChanged();
    return noErr;
}
//-----------------------------------------------------------------------------------
//	ValueChanged
//-----------------------------------------------------------------------------------
OSStatus TValuePictControl::ValueChanged()
{
    if (mImage)
	CGImageRelease(mImage);

    SInt32 value = GetValue();
    CFNumberRef number = CFNumberCreate(NULL, kCFNumberIntType, &value);
    if (CFDictionaryGetValueIfPresent(mImageCache, number, &mImage)) {
	CFRetain(mImage);
    } else if (mBundleRef) {
	char buffer[100];
	char name[100];
	CFStringRef fileName;
	CopyControlTitleAsCFString(GetViewRef(), &fileName);
	CFStringGetCString(fileName, buffer, 100, kCFStringEncodingASCII);
	sprintf(name, "%s%ld.png", buffer, GetValue());
	CFStringRef knob = CFStringCreateWithCStringNoCopy(0, name, kCFStringEncodingASCII, 0);

	CFURLRef url = ::CFBundleCopyResourceURL( mBundleRef, knob, NULL, NULL );
	if (url) {
	    mImage = TImageCache::GetImage(url);
	    if (mImage) {
		CFDictionaryAddValue(mImageCache, number, mImage);
	    }
	    CFRelease( url );
	}
    }
    CFRelease(number);
    Invalidate();
    return noErr;
}

//-----------------------------------------------------------------------------------
//	Draw
//-----------------------------------------------------------------------------------
//	The fun part of the control
//
void TValuePictControl::CompatibleDraw(
				     RgnHandle				inLimitRgn,
				     CGContextRef			inContext,
				     bool  inCompositing)
{
#pragma unused( inLimitRgn )
    if (mImage) {
	HIRect bounds = Bounds();
	HIViewDrawCGImage( inContext, &bounds, mImage);
    }
}

//-----------------------------------------------------------------------------------
//	GetRegion
//-----------------------------------------------------------------------------------
//
OSStatus TValuePictControl::GetRegion(
				ControlPartCode		inPart,
				RgnHandle			outRgn )
{
    OSStatus			err = eventNotHandledErr;

    switch ( inPart )
    {
	case kControlContentMetaPart:
	case kControlStructureMetaPart:
	    TRect bounds = Bounds();
	    Rect qdBounds = bounds;
	    RectRgn( outRgn, &qdBounds );
	    err = noErr;
	    break;
    }

    return err;
}
// -----------------------------------------------------------------------------
//	HitTest
// -----------------------------------------------------------------------------
//	Asks your view to return what part of itself (if any) is hit by the point
//	given to it. The point is in VIEW coordinates, so you should get the view
//	rect to do bounds checking.
//
ControlPartCode TValuePictControl::HitTest(
				      const HIPoint&		inWhere )
{
    ControlPartCode		part;

    // is the mouse on the button?
    if( CGRectContainsPoint( Bounds(), inWhere ) )
	part = kControlButtonPart;
    else
	part = kControlNoPart;

    return part;
}
// -----------------------------------------------------------------------------
//	Track
// -----------------------------------------------------------------------------
//	Default tracking method.
//
OSStatus TValuePictControl::Track(
			     TCarbonEvent&		inEvent,
			     ControlPartCode*	outPart )
{
    Point pt, org;
    float deltaX, deltaY;
    int stillTracking = 1;
    SInt32 mini, maxi;

    mini = GetMinimum();
    maxi = GetMaximum();

    GetMouse ( &org );
    while (stillTracking)
    {
	MouseTrackingResult outResult;

	TrackMouseLocation (NULL, &pt, &outResult);
	if ( outResult == kMouseTrackingMouseUp ) {
	    stillTracking = 0;
	} else {
	    deltaX = (float) (pt.h - org.h);
	    deltaY = (float) (pt.v - org.v);

	    SInt32 curVal = GetValue();
	    SInt32 val = SInt32(rint(curVal+(deltaX-deltaY)*(maxi-mini)/100.));
	    if ( val > maxi) val = maxi;
	    if ( val < mini) val = mini;
	    if (val != curVal) {
		SetValue ( val );
		org = pt;
	    }
	}
    }
    *outPart = kControlNoPart;
    return noErr;
}
// -----------------------------------------------------------------------------
UInt32 TValuePictControl::GetBehaviors()
{
    return TView::GetBehaviors() | kControlSupportsEmbedding | kControlHasSpecialBackground;
}